@@ -4,15 +4,14 @@ import json
 import logging
 import re
 import time
-from enum import Enum
 from typing import Any, Dict, Optional, Tuple, Union
+from urllib.parse import urljoin
 
 import pytz
 import requests
 from flask import Response as FlaskResponse
-from moto.apigateway.models import apigateway_backends
+from jsonschema import ValidationError, validate
 from requests.models import Response
-from six.moves.urllib_parse import urljoin
 
 from localstack import config
 from localstack.constants import (
@@ -23,6 +22,7 @@ from localstack.constants import (
     TEST_AWS_ACCOUNT_ID,
 )
 from localstack.services.apigateway import helpers
+from localstack.services.apigateway.context import ApiInvocationContext
 from localstack.services.apigateway.helpers import (
     API_REGIONS,
     PATH_REGEX_AUTHORIZERS,
@@ -46,6 +46,12 @@ from localstack.services.apigateway.helpers import (
     handle_vpc_links,
     make_error_response,
 )
+from localstack.services.apigateway.integration import (
+    RequestTemplates,
+    ResponseTemplates,
+    SnsIntegration,
+    VtlTemplate,
+)
 from localstack.services.awslambda import lambda_api
 from localstack.services.generic_proxy import ProxyListener
 from localstack.services.kinesis import kinesis_listener
@@ -56,7 +62,6 @@ from localstack.utils.aws import aws_responses, aws_stack
 from localstack.utils.aws.aws_responses import (
     LambdaResponse,
     flask_to_requests_response,
-    parse_query_string,
     request_response_stream,
     requests_response,
 )
@@ -64,15 +69,18 @@ from localstack.utils.aws.request_context import MARKER_APIGW_REQUEST_REGION, TH
 from localstack.utils.common import camel_to_snake_case, json_safe, long_uid, to_bytes, to_str
 
 # set up logger
+from localstack.utils.http import add_query_params_to_url
+
 LOG = logging.getLogger(__name__)
 
 # target ARN patterns
-TARGET_REGEX_S3_URI = (
+TARGET_REGEX_PATH_S3_URI = (
     r"^arn:aws:apigateway:[a-zA-Z0-9\-]+:s3:path/(?P<bucket>[^/]+)/(?P<object>.+)$"
 )
-# regex path pattern for user requests
+TARGET_REGEX_ACTION_S3_URI = r"^arn:aws:apigateway:[a-zA-Z0-9\-]+:s3:action/(?:GetObject&Bucket\=(?P<bucket>[^&]+)&Key\=(?P<object>.+))$"
+# regex path pattern for user requests, handles stages like $default
 PATH_REGEX_USER_REQUEST = (
-    r"^/restapis/([A-Za-z0-9_\-]+)/([A-Za-z0-9_\-]+)/%s/(.*)$" % PATH_USER_REQUEST
+    r"^/restapis/([A-Za-z0-9_\\-]+)(?:/([A-Za-z0-9\_($|%%24)\\-]+))?/%s/(.*)$" % PATH_USER_REQUEST
 )
 # URL pattern for invocations
 HOST_REGEX_EXECUTE_API = (
@@ -83,134 +91,10 @@ HOST_REGEX_EXECUTE_API = (
 REQUEST_TIME_DATE_FORMAT = "%d/%b/%Y:%H:%M:%S %z"
 
 
-class ApiGatewayVersion(Enum):
-    V1 = "v1"
-    V2 = "v2"
-
-
-# type definition for data parameters (i.e., invocation payloads)
-InvocationPayload = Union[Dict, str, bytes]
-
-
 class AuthorizationError(Exception):
     pass
 
 
-class ApiInvocationContext:
-    """Represents the context for an incoming API Gateway invocation."""
-
-    # basic (raw) HTTP invocation details (method, path, data, headers)
-    method: str
-    path: str
-    data: InvocationPayload
-    headers: Dict[str, str]
-
-    # invocation context
-    context: Dict[str, Any]
-    # authentication info for this invocation
-    auth_info: Dict[str, Any]
-
-    # target API/resource details extracted from the invocation
-    apigw_version: ApiGatewayVersion
-    api_id: str
-    stage: str
-    region_name: str
-    # resource path, including any path parameter placeholders (e.g., "/my/path/{id}")
-    resource_path: str
-    integration: Dict
-    resource: Dict
-    # Invocation path with query string, e.g., "/my/path?test". Defaults to "path", can be used
-    #  to overwrite the actual API path, in case the path format "../_user_request_/.." is used.
-    _path_with_query_string: str
-
-    # response templates to be applied to the invocation result
-    response_templates: Dict
-
-    # response object
-    response: Response
-
-    def __init__(
-        self,
-        method,
-        path,
-        data,
-        headers,
-        api_id=None,
-        stage=None,
-        context=None,
-        auth_info=None,
-    ):
-        self.method = method
-        self.path = path
-        self.data = data
-        self.headers = headers
-        self.context = {} if context is None else context
-        self.auth_info = {} if auth_info is None else auth_info
-        self.apigw_version = None
-        self.api_id = api_id
-        self.stage = stage
-        self.region_name = None
-        self.integration = None
-        self.resource = None
-        self.resource_path = None
-        self.path_with_query_string = None
-        self.response_templates = {}
-
-    @property
-    def resource_id(self) -> Optional[str]:
-        return (self.resource or {}).get("id")
-
-    @property
-    def invocation_path(self) -> str:
-        """Return the plain invocation path, without query parameters."""
-        path = self.path_with_query_string or self.path
-        return path.split("?")[0]
-
-    @property
-    def path_with_query_string(self) -> str:
-        """Return invocation path with query string - defaults to the value of 'path', unless customized."""
-        return self._path_with_query_string or self.path
-
-    @path_with_query_string.setter
-    def path_with_query_string(self, new_path: str):
-        """Set a custom invocation path with query string (used to handle "../_user_request_/.." paths)."""
-        self._path_with_query_string = new_path
-
-    def query_params(self) -> Dict:
-        """Extract the query parameters from the target URL or path in this request context."""
-        query_string = self.path_with_query_string.partition("?")[2]
-        return parse_query_string(query_string)
-
-    @property
-    def integration_uri(self) -> Optional[str]:
-        integration = self.integration or {}
-        return integration.get("uri") or integration.get("integrationUri")
-
-    @property
-    def auth_context(self) -> Optional[Dict]:
-        if isinstance(self.auth_info, dict):
-            context = self.auth_info.setdefault("context", {})
-            principal = self.auth_info.get("principalId")
-            if principal:
-                context["principalId"] = principal
-            return context
-
-    @property
-    def auth_identity(self) -> Optional[Dict]:
-        if isinstance(self.auth_info, dict):
-            if self.auth_info.get("identity") is None:
-                self.auth_info["identity"] = {}
-            return self.auth_info["identity"]
-
-    def is_websocket_request(self):
-        upgrade_header = str(self.headers.get("upgrade") or "")
-        return upgrade_header.lower() == "websocket"
-
-    def is_v1(self):
-        """Whether this is an API Gateway v1 request"""
-        return self.apigw_version == ApiGatewayVersion.V1
-
-
 class ProxyListenerApiGateway(ProxyListener):
     def forward_request(self, method, path, data, headers):
         invocation_context = ApiInvocationContext(method, path, data, headers)
@@ -238,7 +122,7 @@ class ProxyListenerApiGateway(ProxyListener):
         if re.match(PATH_REGEX_PATH_MAPPINGS, path):
             return handle_base_path_mappings(method, path, data, headers)
 
-        if is_test_invoke_method(method, path):
+        if helpers.is_test_invoke_method(method, path):
             # if call is from test_invoke_api then use http_method to find the integration,
             #   as test_invoke_api makes a POST call to request the test invocation
             match = re.match(PATH_REGEX_TEST_INVOKE_API, path)
@@ -257,6 +141,7 @@ class ProxyListenerApiGateway(ProxyListener):
                 "headers": dict(result.headers),
             }
             return result
+
         return True
 
     def return_response(self, method, path, data, headers, response):
@@ -303,6 +188,80 @@ class ProxyListenerApiGateway(ProxyListener):
             )
 
 
+class RequestValidator:
+    __slots__ = ["context", "apigateway_client"]
+
+    def __init__(self, context: ApiInvocationContext, apigateway_client):
+        self.context = context
+        self.apigateway_client = apigateway_client
+
+    def is_request_valid(self) -> bool:
+        # make all the positive checks first
+        if self.context.resource is None or "resourceMethods" not in self.context.resource:
+            return True
+
+        resource_methods = self.context.resource["resourceMethods"]
+        if self.context.method not in resource_methods:
+            return True
+
+        # check if there is validator for the resource
+        resource = resource_methods[self.context.method]
+        if not (resource.get("requestValidatorId") or "").strip():
+            return True
+
+        # check if there is a validator for this request
+        validator = self.apigateway_client.get_request_validator(
+            restApiId=self.context.api_id, requestValidatorId=resource["requestValidatorId"]
+        )
+        if validator is None:
+            return True
+
+        # are we validating the body?
+        if self.should_validate_body(validator):
+            is_body_valid = self.validate_body(resource)
+            if not is_body_valid:
+                return is_body_valid
+
+        if self.should_validate_request(validator):
+            is_valid_parameters = self.validate_parameters_and_headers(resource)
+            if not is_valid_parameters:
+                return is_valid_parameters
+
+        return True
+
+    def validate_body(self, resource):
+        # we need a model to validate the body
+        if "requestModels" not in resource or not resource["requestModels"]:
+            return False
+
+        schema_name = resource["requestModels"].get(APPLICATION_JSON)
+        model = self.apigateway_client.get_model(
+            restApiId=self.context.api_id,
+            modelName=schema_name,
+        )
+        if not model:
+            return False
+
+        try:
+            validate(instance=json.loads(self.context.data), schema=json.loads(model["schema"]))
+            return True
+        except ValidationError as e:
+            LOG.warning("failed to validate request body", e)
+            return False
+
+    # TODO implement parameters and headers
+    def validate_parameters_and_headers(self, resource):
+        return True
+
+    @staticmethod
+    def should_validate_body(validator):
+        return validator["validateRequestBody"]
+
+    @staticmethod
+    def should_validate_request(validator):
+        return validator.get("validateRequestParameters")
+
+
 # ------------
 # API METHODS
 # ------------
@@ -359,7 +318,9 @@ def update_content_length(response: Response):
         response.headers["Content-Length"] = str(len(response.content))
 
 
-def apply_request_parameters(uri: str, integration: Dict[str, Any], path_params: Dict[str, str]):
+def apply_request_parameters(
+    uri: str, integration: Dict[str, Any], path_params: Dict[str, str], query_params: Dict[str, str]
+):
     request_parameters = integration.get("requestParameters")
     uri = uri or integration.get("uri") or integration.get("integrationUri") or ""
     if request_parameters:
@@ -369,40 +330,15 @@ def apply_request_parameters(uri: str, integration: Dict[str, Any], path_params:
             request_param_value = f"method.request.path.{key}"
             if request_parameters.get(request_param_key, None) == request_param_value:
                 uri = uri.replace(f"{{{key}}}", path_params[key])
-    return uri
 
+    if integration.get("type") != "HTTP_PROXY" and request_parameters:
+        for key in query_params.copy():
+            request_query_key = f"integration.request.querystring.{key}"
+            request_param_val = f"method.request.querystring.{key}"
+            if request_parameters.get(request_query_key, None) != request_param_val:
+                query_params.pop(key)
 
-def apply_template(
-    integration: Dict[str, Any],
-    req_res_type: str,
-    data: InvocationPayload,
-    path_params={},
-    query_params={},
-    headers={},
-    context={},
-):
-    integration_type = integration.get("type") or integration.get("integrationType")
-    if integration_type in ["HTTP", "AWS"]:
-        # apply custom request template
-        content_type = APPLICATION_JSON  # TODO: make configurable!
-        template = integration.get("%sTemplates" % req_res_type, {}).get(content_type)
-        if template:
-            variables = {"context": context or {}}
-            input_ctx = {"body": data}
-
-            def _params(name=None):
-                # See https://docs.aws.amazon.com/apigateway/latest/developerguide/
-                #    api-gateway-mapping-template-reference.html#input-variable-reference
-                # Returns "request parameter from the path, query string, or header value (searched in that order)"
-                combined = {}
-                combined.update(path_params or {})
-                combined.update(query_params or {})
-                combined.update(headers or {})
-                return combined if not name else combined.get(name)
-
-            input_ctx["params"] = _params
-            data = aws_stack.render_velocity_template(template, input_ctx, variables=variables)
-    return data
+    return add_query_params_to_url(uri, query_params)
 
 
 def apply_response_parameters(invocation_context: ApiInvocationContext):
@@ -416,7 +352,7 @@ def apply_response_parameters(invocation_context: ApiInvocationContext):
     return_code = str(response.status_code)
     if return_code not in entries:
         if len(entries) > 1:
-            LOG.info("Found multiple integration response status codes: %s" % entries)
+            LOG.info("Found multiple integration response status codes: %s", entries)
             return response
         return_code = entries[0]
     response_params = int_responses[return_code].get("responseParameters", {})
@@ -514,10 +450,16 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
 
     # run gateway authorizers for this request
     authorize_invocation(invocation_context)
+
     extracted_path, resource = get_target_resource_details(invocation_context)
     if not resource:
         return make_error_response("Unable to find path %s" % invocation_context.path, 404)
 
+    # validate request
+    validator = RequestValidator(invocation_context, aws_stack.connect_to_service("apigateway"))
+    if not validator.is_request_valid():
+        return make_error_response("Invalid request body", 400)
+
     api_key_required = resource.get("resourceMethods", {}).get(method, {}).get("apiKeyRequired")
     if not is_api_key_valid(api_key_required, headers, invocation_context.stage):
         return make_error_response("Access denied - invalid API key", 403)
@@ -525,7 +467,11 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
     integrations = resource.get("resourceMethods", {})
     integration = integrations.get(method, {})
     if not integration:
-        integration = integrations.get("ANY", {})
+        # HttpMethod: '*'
+        # ResourcePath: '/*' - produces 'X-AMAZON-APIGATEWAY-ANY-METHOD'
+        integration = integrations.get("ANY", {}) or integrations.get(
+            "X-AMAZON-APIGATEWAY-ANY-METHOD", {}
+        )
     integration = integration.get("methodIntegration")
     if not integration:
         if method == "OPTIONS" and "Origin" in headers:
@@ -547,15 +493,14 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
     invocation_context.response_templates = response_templates
     invocation_context.integration = integration
 
-    result = invoke_rest_api_integration(invocation_context)
-    return result
+    return invoke_rest_api_integration(invocation_context)
 
 
 def invoke_rest_api_integration(invocation_context: ApiInvocationContext):
     try:
-        response = invoke_rest_api_integration_backend(
-            invocation_context, invocation_context.integration
-        )
+        response = invoke_rest_api_integration_backend(invocation_context)
+        # TODO remove this setter once all the integrations are migrated to the new response
+        #  handling
         invocation_context.response = response
         response = apply_response_parameters(invocation_context)
         return response
@@ -565,19 +510,19 @@ def invoke_rest_api_integration(invocation_context: ApiInvocationContext):
         return make_error_response(msg, 400)
 
 
-def invoke_rest_api_integration_backend(
-    invocation_context: ApiInvocationContext, integration: Dict
-):
+# TODO: refactor this to have a class per integration type to make it easy to
+# test the encapsulated logic
+def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext):
     # define local aliases from invocation context
     invocation_path = invocation_context.path_with_query_string
     method = invocation_context.method
-    path = invocation_context.path
     data = invocation_context.data
     headers = invocation_context.headers
     api_id = invocation_context.api_id
     stage = invocation_context.stage
     resource_path = invocation_context.resource_path
     response_templates = invocation_context.response_templates
+    integration = invocation_context.integration
 
     # extract integration type and path parameters
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
@@ -586,6 +531,7 @@ def invoke_rest_api_integration_backend(
     uri = integration.get("uri") or integration.get("integrationUri") or ""
     try:
         path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
+        invocation_context.path_params = path_params
     except Exception:
         path_params = {}
 
@@ -599,50 +545,26 @@ def invoke_rest_api_integration_backend(
                     uri.split(":lambda:path")[1].split("functions/")[1].split("/invocations")[0]
                 )
 
-            # apply custom request template
-            data_str = data
-            is_base64_encoded = False
-            try:
-                data_str = json.dumps(data) if isinstance(data, (dict, list)) else to_str(data)
-                data_str = apply_template(
-                    integration,
-                    "request",
-                    data_str,
-                    path_params=path_params,
-                    query_params=query_string_params,
-                    headers=headers,
-                )
-            except UnicodeDecodeError:
-                data_str = base64.b64encode(data_str)
-                is_base64_encoded = True
-            except Exception as e:
-                LOG.warning("Unable to convert API Gateway payload to str: %s" % (e))
-                pass
+            invocation_context.context = get_event_request_context(invocation_context)
+            invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
+            request_templates = RequestTemplates()
+            payload = request_templates.render(invocation_context)
 
-            # Sample request context:
-            # https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-create-api-as-simple-proxy-for-lambda.html#api-gateway-create-api-as-simple-proxy-for-lambda-test
-            request_context = get_event_request_context(invocation_context)
-            stage_variables = (
-                get_stage_variables(api_id, stage)
-                if not is_test_invoke_method(method, path)
-                else None
-            )
             # TODO: change this signature to InvocationContext as well!
             result = lambda_api.process_apigateway_invocation(
                 func_arn,
                 relative_path,
-                data_str,
+                payload,
                 stage,
                 api_id,
                 headers,
-                is_base64_encoded=is_base64_encoded,
+                is_base64_encoded=invocation_context.is_data_base64_encoded,
                 path_params=path_params,
                 query_string_params=query_string_params,
                 method=method,
                 resource_path=resource_path,
-                request_context=request_context,
-                event_context=invocation_context.context,
-                stage_variables=stage_variables,
+                request_context=invocation_context.context,
+                stage_variables=invocation_context.stage_variables,
             )
 
             if isinstance(result, FlaskResponse):
@@ -670,20 +592,21 @@ def invoke_rest_api_integration_backend(
                             body_bytes = base64.b64decode(body_bytes)
                         response._content = body_bytes
                 except Exception as e:
-                    LOG.warning("Couldn't set Lambda response content: %s" % e)
+                    LOG.warning("Couldn't set Lambda response content: %s", e)
                     response._content = "{}"
                 update_content_length(response)
                 response.multi_value_headers = parsed_result.get("multiValueHeaders") or {}
 
             # apply custom response template
-            response._content = apply_template(integration, "response", response._content)
-            response.headers["Content-Length"] = str(len(response.content or ""))
+            invocation_context.response = response
 
-            return response
+            response_templates = ResponseTemplates()
+            response_templates.render(invocation_context)
+            invocation_context.response.headers["Content-Length"] = str(len(response.content or ""))
+            return invocation_context.response
 
         raise Exception(
-            'API Gateway integration type "%s", action "%s", method "%s" invalid or not yet implemented'
-            % (integration_type, uri, method)
+            f'API Gateway integration type "{integration_type}", action "{uri}", method "{method}"'
         )
 
     elif integration_type == "AWS":
@@ -696,39 +619,45 @@ def invoke_rest_api_integration_backend(
                 target = kinesis_listener.ACTION_LIST_STREAMS
             else:
                 LOG.info(
-                    "Unexpected API Gateway integration URI '%s' for integration type %s",
-                    uri,
-                    integration_type,
+                    f"Unexpected API Gateway integration URI '{uri}' for integration type {integration_type}",
                 )
                 target = ""
 
-            # apply request templates
-            new_data = apply_request_response_templates(
-                data, integration.get("requestTemplates"), content_type=APPLICATION_JSON
-            )
+            try:
+                invocation_context.context = get_event_request_context(invocation_context)
+                invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
+                request_templates = RequestTemplates()
+                payload = request_templates.render(invocation_context)
+
+            except Exception as e:
+                LOG.warning("Unable to convert API Gateway payload to str", e)
+                raise
+
             # forward records to target kinesis stream
-            headers = aws_stack.mock_aws_request_headers(service="kinesis")
+            headers = aws_stack.mock_aws_request_headers(
+                service="kinesis", region_name=invocation_context.region_name
+            )
             headers["X-Amz-Target"] = target
+
             result = common.make_http_request(
-                url=config.TEST_KINESIS_URL, method="POST", data=new_data, headers=headers
+                url=config.service_url("kineses"), data=payload, headers=headers, method="POST"
             )
+
             # apply response template
-            result = apply_request_response_templates(
-                result, response_templates, content_type=APPLICATION_JSON
-            )
-            return result
+            invocation_context.response = result
+            response_templates = ResponseTemplates()
+            response_templates.render(invocation_context)
+            return invocation_context.response
 
         elif "states:action/" in uri:
             action = uri.split("/")[-1]
 
             if APPLICATION_JSON in integration.get("requestTemplates", {}):
-                payload = apply_request_response_templates(
-                    data,
-                    integration.get("requestTemplates"),
-                    content_type=APPLICATION_JSON,
-                    as_json=True,
-                )
+                request_templates = RequestTemplates()
+                payload = request_templates.render(invocation_context)
+                payload = json.loads(payload)
             else:
+                # XXX decoding in py3 sounds wrong, this actually might break
                 payload = json.loads(data.decode("utf-8"))
             client = aws_stack.connect_to_service("stepfunctions")
 
@@ -768,14 +697,25 @@ def invoke_rest_api_integration_backend(
                 response = requests_response(content=result)
 
             # apply response templates
-            response = apply_request_response_templates(
-                response, response_templates, content_type=APPLICATION_JSON
-            )
+            invocation_context.response = response
+            response_templates = ResponseTemplates()
+            response_templates.render(invocation_context)
+            # response = apply_request_response_templates(
+            #     response, response_templates, content_type=APPLICATION_JSON
+            # )
             return response
-
-        elif "s3:path/" in uri and method == "GET":
+        # https://docs.aws.amazon.com/apigateway/api-reference/resource/integration/
+        elif ("s3:path/" in uri or "s3:action/" in uri) and method == "GET":
             s3 = aws_stack.connect_to_service("s3")
-            uri_match = re.match(TARGET_REGEX_S3_URI, uri)
+            uri = apply_request_parameters(
+                uri,
+                integration=integration,
+                path_params=path_params,
+                query_params=query_string_params,
+            )
+            uri_match = re.match(TARGET_REGEX_PATH_S3_URI, uri) or re.match(
+                TARGET_REGEX_ACTION_S3_URI, uri
+            )
             if uri_match:
                 bucket, object_key = uri_match.group("bucket", "object")
                 LOG.debug("Getting request for bucket %s object %s", bucket, object_key)
@@ -804,18 +744,30 @@ def invoke_rest_api_integration_backend(
                 template = integration["requestTemplates"][APPLICATION_JSON]
                 account_id, queue = uri.split("/")[-2:]
                 region_name = uri.split(":")[3]
-
-                new_request = "%s&QueueName=%s" % (
-                    aws_stack.render_velocity_template(template, data),
-                    queue,
-                )
+                if "GetQueueUrl" in template or "CreateQueue" in template:
+                    request_templates = RequestTemplates()
+                    payload = request_templates.render(invocation_context)
+                    new_request = f"{payload}&QueueName={queue}"
+                else:
+                    request_templates = RequestTemplates()
+                    payload = request_templates.render(invocation_context)
+                    queue_url = f"{config.get_edge_url()}/{account_id}/{queue}"
+                    new_request = f"{payload}&QueueUrl={queue_url}"
                 headers = aws_stack.mock_aws_request_headers(service="sqs", region_name=region_name)
 
-                url = urljoin(config.TEST_SQS_URL, "%s/%s" % (TEST_AWS_ACCOUNT_ID, queue))
+                url = urljoin(config.service_url("sqs"), f"{TEST_AWS_ACCOUNT_ID}/{queue}")
                 result = common.make_http_request(
                     url, method="POST", headers=headers, data=new_request
                 )
                 return result
+            elif uri.startswith("arn:aws:apigateway:") and ":sns:path" in uri:
+                invocation_context.context = get_event_request_context(invocation_context)
+                invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
+
+                integration_response = SnsIntegration().invoke(invocation_context)
+                return apply_request_response_templates(
+                    integration_response, response_templates, content_type=APPLICATION_JSON
+                )
 
         raise Exception(
             'API Gateway AWS integration action URI "%s", method "%s" not yet implemented'
@@ -833,7 +785,7 @@ def invoke_rest_api_integration_backend(
 
                 if response_template is None:
                     msg = "Invalid response template defined in integration response."
-                    LOG.info("%s Existing: %s" % (msg, response_templates))
+                    LOG.info("%s Existing: %s", msg, response_templates)
                     return make_error_response(msg, 404)
 
                 response_template = json.loads(response_template)
@@ -870,18 +822,33 @@ def invoke_rest_api_integration_backend(
                 uri = "http://%s/%s" % (instance["Id"], invocation_path.lstrip("/"))
 
         # apply custom request template
-        data = apply_template(integration, "request", data)
-        if isinstance(data, dict):
-            data = json.dumps(data)
-        uri = apply_request_parameters(uri, integration=integration, path_params=path_params)
-        result = requests.request(method=method, url=uri, data=data, headers=headers)
+        invocation_context.context = get_event_request_context(invocation_context)
+        invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
+        request_templates = RequestTemplates()
+        payload = request_templates.render(invocation_context)
+
+        if isinstance(payload, dict):
+            payload = json.dumps(payload)
+        uri = apply_request_parameters(
+            uri, integration=integration, path_params=path_params, query_params=query_string_params
+        )
+        result = requests.request(method=method, url=uri, data=payload, headers=headers)
         # apply custom response template
-        result = apply_template(integration, "response", result)
-        return result
+        invocation_context.response = result
+        response_templates = ResponseTemplates()
+        response_templates.render(invocation_context)
+        return invocation_context.response
 
     elif integration_type == "MOCK":
-        # return empty response - details filled in via responseParameters above...
-        return requests_response({})
+
+        # TODO: apply tell don't ask principle inside ResponseTemplates or InvocationContext
+        invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
+        invocation_context.response = requests_response({})
+
+        response_templates = ResponseTemplates()
+        response_templates.render(invocation_context)
+
+        return invocation_context.response
 
     if method == "OPTIONS":
         # fall back to returning CORS headers if this is an OPTIONS request
@@ -895,10 +862,13 @@ def invoke_rest_api_integration_backend(
 
 def get_target_resource_details(invocation_context: ApiInvocationContext) -> Tuple[str, Dict]:
     """Look up and return the API GW resource (path pattern + resource dict) for the given invocation context."""
-    path_map = helpers.get_rest_api_paths(rest_api_id=invocation_context.api_id)
+    path_map = helpers.get_rest_api_paths(
+        rest_api_id=invocation_context.api_id, region_name=invocation_context.region_name
+    )
     relative_path = invocation_context.invocation_path
     try:
         extracted_path, resource = get_resource_for_path(path=relative_path, path_map=path_map)
+        invocation_context.resource = resource
         return extracted_path, resource
     except Exception:
         return None, None
@@ -911,17 +881,7 @@ def get_target_resource_method(invocation_context: ApiInvocationContext) -> Opti
         return None
     methods = resource.get("resourceMethods") or {}
     method_name = invocation_context.method.upper()
-    method_details = methods.get(method_name) or methods.get("ANY")
-    return method_details
-
-
-def get_stage_variables(api_id: str, stage: str) -> Dict[str, str]:
-    if not stage:
-        return {}
-    region_name = [name for name, region in apigateway_backends.items() if api_id in region.apis][0]
-    api_gateway_client = aws_stack.connect_to_service("apigateway", region_name=region_name)
-    response = api_gateway_client.get_stage(restApiId=api_id, stageName=stage)
-    return response.get("variables")
+    return methods.get(method_name) or methods.get("ANY")
 
 
 def get_event_request_context(invocation_context: ApiInvocationContext):
@@ -945,11 +905,11 @@ def get_event_request_context(invocation_context: ApiInvocationContext):
     account_id = account_id or TEST_AWS_ACCOUNT_ID
     domain_name = f"{api_id}.execute-api.{LOCALHOST_HOSTNAME}"
     request_context = {
-        "resourcePath": resource_path or relative_path,
+        "accountId": account_id,
         "apiId": api_id,
+        "resourcePath": resource_path or relative_path,
         "domainPrefix": api_id,
         "domainName": domain_name,
-        "accountId": account_id,
         "resourceId": resource_id,
         "requestId": long_uid(),
         "identity": {
@@ -963,6 +923,7 @@ def get_event_request_context(invocation_context: ApiInvocationContext):
             REQUEST_TIME_DATE_FORMAT
         ),
         "requestTimeEpoch": int(time.time() * 1000),
+        "authorizer": {},
     }
 
     # set "authorizer" and "identity" event attributes from request context
@@ -971,7 +932,7 @@ def get_event_request_context(invocation_context: ApiInvocationContext):
         request_context["authorizer"] = auth_context
     request_context["identity"].update(invocation_context.auth_identity or {})
 
-    if not is_test_invoke_method(method, path):
+    if not helpers.is_test_invoke_method(method, path):
         request_context["path"] = (f"/{stage}" if stage else "") + relative_path
         request_context["stage"] = stage
     return request_context
@@ -992,7 +953,7 @@ def apply_request_response_templates(
     if not template:
         return data
     content = (data.content if is_response else data) or ""
-    result = aws_stack.render_velocity_template(template, content, as_json=as_json)
+    result = VtlTemplate().render_vtl(template, content, as_json=as_json)
     if is_response:
         data._content = result
         update_content_length(data)
@@ -1000,9 +961,5 @@ def apply_request_response_templates(
     return result
 
 
-def is_test_invoke_method(method, path):
-    return method == "POST" and bool(re.match(PATH_REGEX_TEST_INVOKE_API, path))
-
-
 # instantiate listener
 UPDATE_APIGATEWAY = ProxyListenerApiGateway()
