@@ -9,6 +9,7 @@ import shutil
 import stat
 import sys
 import tempfile
+import threading
 import time
 from pathlib import Path
 from typing import Callable, Dict, List, Tuple
@@ -17,7 +18,7 @@ import requests
 from plugin import Plugin, PluginManager
 
 from localstack import config
-from localstack.config import dirs, is_env_true
+from localstack.config import dirs
 from localstack.constants import (
     DEFAULT_SERVICE_PORTS,
     DYNAMODB_JAR_URL,
@@ -28,29 +29,27 @@ from localstack.constants import (
     KMS_URL_PATTERN,
     LOCALSTACK_MAVEN_VERSION,
     MODULE_MAIN_PATH,
+    OPENSEARCH_DEFAULT_VERSION,
     STS_JAR_URL,
 )
 from localstack.runtime import hooks
-from localstack.utils.common import (
+from localstack.utils.archives import untar, unzip
+from localstack.utils.docker_utils import DOCKER_CLIENT
+from localstack.utils.files import (
     chmod_r,
-    download,
     file_exists_not_empty,
-    get_arch,
-    get_os,
-    is_windows,
     load_file,
     mkdir,
     new_tmp_file,
-    parallelize,
-    retry,
+    replace_in_file,
     rm_rf,
-    run,
-    safe_run,
     save_file,
-    untar,
-    unzip,
 )
-from localstack.utils.docker_utils import DOCKER_CLIENT
+from localstack.utils.http import download
+from localstack.utils.platform import get_arch, is_windows
+from localstack.utils.run import run
+from localstack.utils.sync import retry
+from localstack.utils.threads import parallelize
 
 LOG = logging.getLogger(__name__)
 
@@ -60,6 +59,7 @@ INSTALL_DIR_KCL = "%s/amazon-kinesis-client" % dirs.static_libs
 INSTALL_DIR_STEPFUNCTIONS = "%s/stepfunctions" % dirs.static_libs
 INSTALL_DIR_KMS = "%s/kms" % dirs.static_libs
 INSTALL_DIR_ELASTICMQ = "%s/elasticmq" % dirs.static_libs
+INSTALL_DIR_KINESIS_MOCK = os.path.join(dirs.static_libs, "kinesis-mock")
 INSTALL_PATH_LOCALSTACK_FAT_JAR = "%s/localstack-utils-fat.jar" % dirs.static_libs
 INSTALL_PATH_DDB_JAR = os.path.join(INSTALL_DIR_DDB, "DynamoDBLocal.jar")
 INSTALL_PATH_KCL_JAR = os.path.join(INSTALL_DIR_KCL, "aws-java-sdk-sts.jar")
@@ -67,17 +67,16 @@ INSTALL_PATH_STEPFUNCTIONS_JAR = os.path.join(INSTALL_DIR_STEPFUNCTIONS, "StepFu
 INSTALL_PATH_KMS_BINARY_PATTERN = os.path.join(INSTALL_DIR_KMS, "local-kms.<arch>.bin")
 INSTALL_PATH_ELASTICMQ_JAR = os.path.join(INSTALL_DIR_ELASTICMQ, "elasticmq-server.jar")
 INSTALL_PATH_KINESALITE_CLI = os.path.join(INSTALL_DIR_NPM, "kinesalite", "cli.js")
-INSTALL_PATH_KINESIS_MOCK = os.path.join(dirs.static_libs, "kinesis-mock")
 URL_LOCALSTACK_FAT_JAR = (
     "https://repo1.maven.org/maven2/"
     + "cloud/localstack/localstack-utils/{v}/localstack-utils-{v}-fat.jar"
 ).format(v=LOCALSTACK_MAVEN_VERSION)
 
 MARKER_FILE_LIGHT_VERSION = "%s/.light-version" % dirs.static_libs
-IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local"
+IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local:1.7.9"
 ARTIFACTS_REPO = "https://github.com/localstack/localstack-artifacts"
 SFN_PATCH_URL_PREFIX = (
-    f"{ARTIFACTS_REPO}/raw/047cc6dcd2e31f5ff3ec52d293c61b875f606958/stepfunctions-local-patch"
+    f"{ARTIFACTS_REPO}/raw/2958554c8aeadff0e8f5d0e35f6e520d834854ea/stepfunctions-local-patch"
 )
 SFN_PATCH_CLASS1 = "com/amazonaws/stepfunctions/local/runtime/Config.class"
 SFN_PATCH_CLASS2 = (
@@ -85,8 +84,15 @@ SFN_PATCH_CLASS2 = (
 )
 SFN_PATCH_CLASS_STARTER = "cloud/localstack/StepFunctionsStarter.class"
 SFN_PATCH_CLASS_REGION = "cloud/localstack/RegionAspect.class"
+SFN_PATCH_CLASS_ASYNC2SERVICEAPI = "cloud/localstack/Async2ServiceApi.class"
+SFN_PATCH_CLASS_DESCRIBEEXECUTIONPARSED = "cloud/localstack/DescribeExecutionParsed.class"
 SFN_PATCH_FILE_METAINF = "META-INF/aop.xml"
 
+SFN_AWS_SDK_URL_PREFIX = (
+    f"{ARTIFACTS_REPO}/raw/a4adc8f4da9c7ec0d93b50ca5b73dd14df791c0e/stepfunctions-internal-awssdk"
+)
+SFN_AWS_SDK_LAMBDA_ZIP_FILE = f"{SFN_AWS_SDK_URL_PREFIX}/awssdk.zip"
+
 # additional JAR libs required for multi-region and persistence (PRO only) support
 MAVEN_REPO = "https://repo1.maven.org/maven2"
 URL_ASPECTJRT = f"{MAVEN_REPO}/org/aspectj/aspectjrt/1.9.7/aspectjrt-1.9.7.jar"
@@ -94,7 +100,7 @@ URL_ASPECTJWEAVER = f"{MAVEN_REPO}/org/aspectj/aspectjweaver/1.9.7/aspectjweaver
 JAR_URLS = [URL_ASPECTJRT, URL_ASPECTJWEAVER]
 
 # kinesis-mock version
-KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.2.0"
+KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.2.2"
 KINESIS_MOCK_RELEASE_URL = (
     "https://api.github.com/repos/etspaceman/kinesis-mock/releases/tags/" + KINESIS_MOCK_VERSION
 )
@@ -116,8 +122,8 @@ GO_INSTALL_FOLDER = os.path.join(config.dirs.var_libs, "awslamba-go-runtime")
 GO_LAMBDA_RUNTIME = os.path.join(GO_INSTALL_FOLDER, "aws-lambda-mock")
 GO_LAMBDA_MOCKSERVER = os.path.join(GO_INSTALL_FOLDER, "mockserver")
 
-# Terraform (used for tests, whose templates require TF < 0.14.0 )
-TERRAFORM_VERSION = "0.13.7"
+# Terraform (used for tests)
+TERRAFORM_VERSION = "1.1.3"
 TERRAFORM_URL_TEMPLATE = (
     "https://releases.hashicorp.com/terraform/{version}/terraform_{version}_{os}_{arch}.zip"
 )
@@ -130,9 +136,11 @@ TEST_LAMBDA_JAR_URL = "{url}/cloud/localstack/{name}/{version}/{name}-{version}-
     version=LOCALSTACK_MAVEN_VERSION, url=MAVEN_BASE_URL, name="localstack-utils"
 )
 
+OS_INSTALL_LOCKS = {}
+
 
 def get_elasticsearch_install_version(version: str) -> str:
-    from localstack.services.es import versions
+    from localstack.services.opensearch import versions
 
     if config.SKIP_INFRA_DOWNLOADS:
         return ELASTICSEARCH_DEFAULT_VERSION
@@ -141,20 +149,22 @@ def get_elasticsearch_install_version(version: str) -> str:
 
 
 def get_elasticsearch_install_dir(version: str) -> str:
-    version = get_elasticsearch_install_version(version)
-
-    if version == ELASTICSEARCH_DEFAULT_VERSION and not os.path.exists(MARKER_FILE_LIGHT_VERSION):
+    if version == get_elasticsearch_install_version(
+        ELASTICSEARCH_DEFAULT_VERSION
+    ) and not os.path.exists(MARKER_FILE_LIGHT_VERSION):
         # install the default version into a subfolder of the code base
         install_dir = os.path.join(dirs.static_libs, "elasticsearch")
     else:
         # put all other versions into the TMP_FOLDER
-        install_dir = os.path.join(config.dirs.tmp, "elasticsearch", version)
+        install_dir = os.path.join(config.dirs.var_libs, "elasticsearch", version)
 
     return install_dir
 
 
 def install_elasticsearch(version=None):
-    from localstack.services.es import versions
+    # locally import to avoid having a dependency on ASF when starting the CLI
+    from localstack.aws.api.opensearch import EngineType
+    from localstack.services.opensearch import versions
 
     if not version:
         version = ELASTICSEARCH_DEFAULT_VERSION
@@ -164,7 +174,7 @@ def install_elasticsearch(version=None):
     installed_executable = os.path.join(install_dir, "bin", "elasticsearch")
     if not os.path.exists(installed_executable):
         log_install_msg("Elasticsearch (%s)" % version)
-        es_url = versions.get_download_url(version)
+        es_url = versions.get_download_url(version, EngineType.Elasticsearch)
         install_dir_parent = os.path.dirname(install_dir)
         mkdir(install_dir_parent)
         # download and extract archive
@@ -188,7 +198,8 @@ def install_elasticsearch(version=None):
                 LOG.info("Installing Elasticsearch plugin %s", plugin)
 
                 def try_install():
-                    safe_run([plugin_binary, "install", "-b", plugin])
+                    output = run([plugin_binary, "install", "-b", plugin])
+                    LOG.debug("Plugin installation output: %s", output)
 
                 # We're occasionally seeing javax.net.ssl.SSLHandshakeException -> add download retries
                 download_attempts = 3
@@ -223,6 +234,63 @@ def install_elasticsearch(version=None):
             save_file(jvm_options_file, jvm_options_replaced)
 
 
+def get_opensearch_install_version(version: str) -> str:
+    from localstack.services.opensearch import versions
+
+    if config.SKIP_INFRA_DOWNLOADS:
+        version = OPENSEARCH_DEFAULT_VERSION
+
+    return versions.get_install_version(version)
+
+
+def get_opensearch_install_dir(version: str) -> str:
+    return os.path.join(config.dirs.var_libs, "opensearch", version)
+
+
+def install_opensearch(version=None):
+    # locally import to avoid having a dependency on ASF when starting the CLI
+    from localstack.aws.api.opensearch import EngineType
+    from localstack.services.opensearch import versions
+
+    if not version:
+        version = OPENSEARCH_DEFAULT_VERSION
+
+    version = get_opensearch_install_version(version)
+    install_dir = get_opensearch_install_dir(version)
+    installed_executable = os.path.join(install_dir, "bin", "opensearch")
+    if not os.path.exists(installed_executable):
+        with OS_INSTALL_LOCKS.setdefault(version, threading.Lock()):
+            if not os.path.exists(installed_executable):
+                log_install_msg("OpenSearch (%s)" % version)
+                opensearch_url = versions.get_download_url(version, EngineType.OpenSearch)
+                install_dir_parent = os.path.dirname(install_dir)
+                mkdir(install_dir_parent)
+                # download and extract archive
+                tmp_archive = os.path.join(
+                    config.dirs.tmp, f"localstack.{os.path.basename(opensearch_url)}"
+                )
+                download_and_extract_with_retry(opensearch_url, tmp_archive, install_dir_parent)
+                opensearch_dir = glob.glob(os.path.join(install_dir_parent, "opensearch*"))
+                if not opensearch_dir:
+                    raise Exception("Unable to find OpenSearch folder in %s" % install_dir_parent)
+                shutil.move(opensearch_dir[0], install_dir)
+
+                for dir_name in ("data", "logs", "modules", "plugins", "config/scripts"):
+                    dir_path = os.path.join(install_dir, dir_name)
+                    mkdir(dir_path)
+                    chmod_r(dir_path, 0o777)
+
+    # patch JVM options file - replace hardcoded heap size settings
+    jvm_options_file = os.path.join(install_dir, "config", "jvm.options")
+    if os.path.exists(jvm_options_file):
+        jvm_options = load_file(jvm_options_file)
+        jvm_options_replaced = re.sub(
+            r"(^-Xm[sx][a-zA-Z0-9\.]+$)", r"# \1", jvm_options, flags=re.MULTILINE
+        )
+        if jvm_options != jvm_options_replaced:
+            save_file(jvm_options_file, jvm_options_replaced)
+
+
 def install_sqs_provider():
     if SQS_BACKEND_IMPL == "elasticmq":
         install_elasticmq()
@@ -242,31 +310,53 @@ def install_elasticmq():
 
 def install_kinesis():
     if config.KINESIS_PROVIDER == "kinesalite":
-        return install_kinesalite()
-    elif config.KINESIS_PROVIDER == "kinesis-mock":
-        return install_kinesis_mock()
-    else:
-        raise ValueError("unknown kinesis provider %s" % config.KINESIS_PROVIDER)
+        install_kinesalite()
+        return
+    if config.KINESIS_PROVIDER == "kinesis-mock":
+        is_installed, bin_path = get_is_kinesis_mock_installed()
+        if not is_installed:
+            install_kinesis_mock(bin_path)
+        return
+    raise ValueError("unknown kinesis provider %s" % config.KINESIS_PROVIDER)
+
+
+def _apply_patches_kinesalite():
+    files = [
+        "%s/kinesalite/validations/decreaseStreamRetentionPeriod.js",
+        "%s/kinesalite/validations/increaseStreamRetentionPeriod.js",
+    ]
+    for file_path in files:
+        file_path = file_path % INSTALL_DIR_NPM
+        replace_in_file("lessThanOrEqual: 168", "lessThanOrEqual: 8760", file_path)
 
 
 def install_kinesalite():
     if not os.path.exists(INSTALL_PATH_KINESALITE_CLI):
         log_install_msg("Kinesis")
         run('cd "%s" && npm install' % MODULE_MAIN_PATH)
+        _apply_patches_kinesalite()
 
 
-def install_kinesis_mock():
-    target_dir = INSTALL_PATH_KINESIS_MOCK
+def get_is_kinesis_mock_installed() -> Tuple[bool, str]:
+    """
+    Checks the host system to see if kinesis mock is installed and where.
+    :returns: True if kinesis mock is installed (False otherwise) and the expected installation path
+    """
+    bin_file_path = kinesis_mock_install_path()
+    if os.path.exists(bin_file_path):
+        LOG.debug("kinesis-mock found at %s", bin_file_path)
+        return True, bin_file_path
+    return False, bin_file_path
 
+
+def kinesis_mock_install_path() -> str:
     machine = platform.machine().lower()
     system = platform.system().lower()
     version = platform.version().lower()
-
     is_probably_m1 = system == "darwin" and ("arm64" in version or "arm32" in version)
 
     LOG.debug("getting kinesis-mock for %s %s", system, machine)
-
-    if is_env_true("KINESIS_MOCK_FORCE_JAVA"):
+    if config.is_env_true("KINESIS_MOCK_FORCE_JAVA"):
         # sometimes the static binaries may have problems, and we want to fal back to Java
         bin_file = "kinesis-mock.jar"
     elif (machine == "x86_64" or machine == "amd64") and not is_probably_m1:
@@ -281,44 +371,44 @@ def install_kinesis_mock():
     else:
         bin_file = "kinesis-mock.jar"
 
-    bin_file_path = os.path.join(target_dir, bin_file)
-    if os.path.exists(bin_file_path):
-        LOG.debug("kinesis-mock found at %s", bin_file_path)
-        return bin_file_path
+    bin_file_path = os.path.join(INSTALL_DIR_KINESIS_MOCK, bin_file)
+    return bin_file_path
 
+
+def install_kinesis_mock(bin_file_path: str = None):
     response = requests.get(KINESIS_MOCK_RELEASE_URL)
     if not response.ok:
         raise ValueError(
             "Could not get list of releases from %s: %s" % (KINESIS_MOCK_RELEASE_URL, response.text)
         )
 
+    bin_file_path = bin_file_path or kinesis_mock_install_path()
     github_release = response.json()
     download_url = None
+    bin_file_name = os.path.basename(bin_file_path)
     for asset in github_release.get("assets", []):
         # find the correct binary in the release
-        if asset["name"] == bin_file:
+        if asset["name"] == bin_file_name:
             download_url = asset["browser_download_url"]
             break
-
     if download_url is None:
         raise ValueError(
-            "could not find required binary %s in release %s" % (bin_file, KINESIS_MOCK_RELEASE_URL)
+            "could not find required binary %s in release %s"
+            % (bin_file_name, KINESIS_MOCK_RELEASE_URL)
         )
 
-    mkdir(target_dir)
+    mkdir(INSTALL_DIR_KINESIS_MOCK)
     LOG.info("downloading kinesis-mock binary from %s", download_url)
     download(download_url, bin_file_path)
     chmod_r(bin_file_path, 0o777)
-    return bin_file_path
 
 
 def install_local_kms():
-    local_arch = get_os()
+    local_arch = f"{platform.system().lower()}-{get_arch()}"
     binary_path = INSTALL_PATH_KMS_BINARY_PATTERN.replace("<arch>", local_arch)
     if not os.path.exists(binary_path):
         log_install_msg("KMS")
         mkdir(INSTALL_DIR_KMS)
-        # TODO ARM download platform specific binary
         kms_url = KMS_URL_PATTERN.replace("<arch>", local_arch)
         download(kms_url, binary_path)
         chmod_r(binary_path, 0o777)
@@ -327,7 +417,10 @@ def install_local_kms():
 def install_stepfunctions_local():
     if not os.path.exists(INSTALL_PATH_STEPFUNCTIONS_JAR):
         # pull the JAR file from the Docker image, which is more up-to-date than the downloadable JAR file
-        # TODO: works only when running on the host, outside of Docker -> add a fallback if running in Docker?
+        if not DOCKER_CLIENT.has_docker():
+            # TODO: works only when a docker socket is available -> add a fallback if running without Docker?
+            LOG.warning("Docker not available - skipping installation of StepFunctions dependency")
+            return
         log_install_msg("Step Functions")
         mkdir(INSTALL_DIR_STEPFUNCTIONS)
         DOCKER_CLIENT.pull_image(IMAGE_NAME_SFN_LOCAL)
@@ -355,6 +448,8 @@ def install_stepfunctions_local():
         SFN_PATCH_CLASS2,
         SFN_PATCH_CLASS_REGION,
         SFN_PATCH_CLASS_STARTER,
+        SFN_PATCH_CLASS_ASYNC2SERVICEAPI,
+        SFN_PATCH_CLASS_DESCRIBEEXECUTIONPARSED,
         SFN_PATCH_FILE_METAINF,
     ]
     for patch_class in classes:
@@ -382,6 +477,11 @@ def install_stepfunctions_local():
         if not file_exists_not_empty(target):
             download(jar_url, target)
 
+    # download aws-sdk lambda handler
+    target = os.path.join(INSTALL_DIR_STEPFUNCTIONS, "localstack-internal-awssdk", "awssdk.zip")
+    if not file_exists_not_empty(target):
+        download(SFN_AWS_SDK_LAMBDA_ZIP_FILE, target)
+
 
 def add_file_to_jar(class_file, class_url, target_jar, base_dir=None):
     base_dir = base_dir or os.path.dirname(target_jar)
@@ -558,10 +658,7 @@ def log_install_msg(component, verbatim=False):
 def download_and_extract(archive_url, target_dir, retries=0, sleep=3, tmp_archive=None):
     mkdir(target_dir)
 
-    if tmp_archive:
-        _, ext = os.path.splitext(tmp_archive)
-    else:
-        _, ext = os.path.splitext(archive_url)
+    _, ext = os.path.splitext(tmp_archive or archive_url)
 
     tmp_archive = tmp_archive or new_tmp_file()
     if not os.path.exists(tmp_archive) or os.path.getsize(tmp_archive) <= 0:
@@ -576,10 +673,10 @@ def download_and_extract(archive_url, target_dir, retries=0, sleep=3, tmp_archiv
 
     if ext == ".zip":
         unzip(tmp_archive, target_dir)
-    elif ext == ".gz" or ext == ".bz2":
+    elif ext in [".bz2", ".gz", ".tgz"]:
         untar(tmp_archive, target_dir)
     else:
-        raise Exception("Unsupported archive format: %s" % ext)
+        raise Exception(f"Unsupported archive format: {ext}")
 
 
 def download_and_extract_with_retry(archive_url, tmp_archive, target_dir):
@@ -622,6 +719,7 @@ class CommunityInstallerRepository(InstallerRepository):
             ("dynamodb-local", install_dynamodb_local),
             ("elasticmq", install_elasticmq),
             ("elasticsearch", install_elasticsearch),
+            ("opensearch", install_opensearch),
             ("kinesalite", install_kinesalite),
             ("kinesis-client-libs", install_amazon_kinesis_client_libs),
             ("kinesis-mock", install_kinesis_mock),
